//08-06-2015
//Программа для исследования системы длинных джозефсоновских переходов с емкостной (CCJJ+DC)
//и индуктивной связью 
//Решается следующая система уравнений:
//dph_i=Summa(C_i_k*V_k)
//dV_i=Summa(inversL_i_k*ph_xx)-beta*V_i-sin(ph_i)+I
//решается методом Рунге Кутта четвертого порядка
//Усреднение проводиться сначала по координате затем по времени
//При усреднении интегрирование по координате проводится по методу Симпсона
//а по времени по методу прямоугольников 
#include <mpi.h>
#include<iostream>
#include<math.h>
#include<iomanip>
#include<stdio.h>
#include<stdlib.h>
#include <fstream>
#include <ctime>
#define comm MPI_COMM_WORLD

using namespace std;
int main(int argc, char* argv[]){ // начальное время
	int	N=10;	//Количество контактов
	double
		//При значениях тока Ix записывается координатная зависимость разность H,ph и V
		Ix[]={0.35, 0.288};
	char c='p'; 	//В случае периодических граничных условий вставте 'p' а не периодических 'n'
	char str[50];//переменное для хранения названия файлов учитывающее номер Джозефсоновского перехода
	int a=1,//параметр определения направление тока
		i,	j,	k,//индексы для работы с матрицами
		t,	x,	//индекс по времени и по координате
		tN, xN,	//количество точек по времени и по координате
		ti,//номер цикла по времени от которого начинается интегрирование по времени
		f,	g,	nt,	nx,	F,	G,
		//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		deltaT=5,//Шаг по времени для записи пространственно-временной зависимости
		deltaX=2,//Шаг по координате для записи пространственно-врепенной зависимости
		at=-1,//Направление тока для записи временных зависимостей
		ax=-1,//Направление тока для записи пространственно-временных зависимостей
		capacitive=1,//При учете емкостной связи вставьте "capacitive=1" 
		inductive=1,//При учете индуктивной связи вставьте "inductive=1"
		time_dep=0,	//Если необходимо вычислить временную зависимость заряда
					//и напряжения, вставьте  "time_dep=1"
		currents=0,//Если необходимо вычислить токи вставьте "currents=1"
		spatiotemporal_dep=0,	//Если необходимо вычислить координатную зависимость напряжении 
							//и разность фаз, вставте  "spatiotemporal_dep=1"
		init_condition=0;	//Если начальные условия отличны от нуля вставте "init_condition=0"
							//Если "init_condition=1" то у вас должен быт файл "initial.dat"
	//----------------------------------------------------------------------------------
	//Переменные для MPI

	int rank,np, delta, itt;
    MPI_Status status;
    MPI_Init(&argc,&argv);
    MPI_Comm_rank(comm,&rank);
    MPI_Comm_size(comm,&np);
	int xmin,xmax;
	double starttime, endtime;
	//
	
	double
		Ti=5,//Начало интервала интегрирования по времени
		Tf0=100,//Максимальное значения времени
		Tf1=100,//Максимальное значения времени в интервале [Itime1-|-Itime2]
		JP0=0.005,//Шаг по току (0.01)
		JP1=0.005,//Шаг по току в интервале {Imore1-Imore2}
		length=10,//Длина контакта
		XP=0.05,//Шаг по координате
		TP=XP/5,//Шаг по времени
		I0=0.01,//Начальное значения тока
		Hext=0,//Значение внещного магнитного поля
		Imax=1.1,//Максимальное значения тока
		Imore1=0.15,//Начало интервала при которой меняется Шаг по току
		Imore2=0.95,//Конец интервала при которой меняется Шаг по току
		noismax=0,//Максимальная амплитуда шума
		DC=1.1,//Диагональные элементы матрицы емкостной связи
		SC=-0.05,//Элементы матрицы емкостной связи
		S=-0.05,//Диагональные элементы матрицы индуктивной связи
		beta=0.1,//Параметр диссипации
		Itime1[]={0.4},//Интервал для записи временной зависимости (усредненной по координате)
		Itime2[]={0.5},
		Q0=1,//Амплитуда заряда в сверхпроводящих слоях
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		r,pr,c1,
		time,//Значение времени для записи
		xrecord,//Значение координаты для записи
		C[N+2][N+2],//Матрица емкостьной связи
		L[N+2][N+2],//Матрица индуктивной связи
		L1[N+2][N+2],//Копия матрицы L для нахождения обратной матрицы т.е. L^(-1)
		U[N+2][N+2],//Единичная матрица
		inversL[N+2][N+2],//Обратная матрица к матрице L
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		int_Is_x[N+2],//Интеграл от сверхпроводящего тока по координате при фиксированном времени
		int_Iqp_x[N+2],//Интеграл от квазичастичного тока по координате при фиксированном времени
		int_Idisp_x[N+2],//Интеграл от тока смещения по координате при фиксированном времени
		int_Idif_x[N+2],//Интеграл от диффузионного тока по координате при фиксированном времени
		int_dpht_x[N+2],
		
		Isav_x[N+2],//Значение сверхпроводящего тока усредненное по координате
		Iqpav_x[N+2],//Значение квазичастичного тока усредненное по координате 
		Idispav_x[N+2],//Значение тока смещения усредненное по координате 
		Idifav_x[N+2],//Значение диффузионного тока усредненное по координате
		dphtav_x[N+2],

		int_Isav_t[N+2],//Интеграл по времени от сверхпроводящего тока (усредненного по координате)
		int_Iqpav_t[N+2],//Интеграл по времени от квазичастичного тока (усредненного по координате)
		int_Idispav_t[N+2],//Интеграл по времени от тока смещения (усредненного по координате)
		int_Idifav_t[N+2],//Интеграл по времени от диффузионного тока (усредненного по координате)
		int_dphtav_t[N+2],

		Isav[N+2],//Среднее значения сверхпроводящего тока	
		Iqpav[N+2],//Среднее значения квазичастичного тока
		Idispav[N+2],//Среднее значения тока смещения
		Idifav[N+2],//Среднее значения диффузионного тока
		dphtav[N+2],

		Isum[N+2],//Сумма токов (параметр вводился для проверки расчета, если Isum=I то результат правильно)		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		int_Q_x[N+2],//Интеграл от заряда в S-слоях по координате при фиксированном времени
		Qav_x[N+2],//Значение заряда в S-слоях усредненное по координате

		Irand[N+2],
		
		int_V_x[N+2],//Интеграл от напряжения по координате при фиксированном времени
		Vav_x[N+2],//Значение напряжения усредненное по координате
		int_Vav_t[N+2],//Интеграл по времени от напряжения (усредненного по координате)
		Vav[N+2],//Среднее значение напряжения в отдельном контакте
		v,//Сумма средных значений напряжений во всех ДП
		I,JP,Tf;
		
		N=atoi(argv[1]);
		length=atof(argv[2]);
		DC=atof(argv[3]);
		SC=atof(argv[4]);
		
		printf("---%d---%f---%f---%f\n",N, length, DC, SC);
		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	I=I0; JP=JP0; Tf=Tf0; xN = 1 + length / XP;
	tN = Tf / TP; ti=tN*Ti/Tf; c1=1/pow(XP,2);
	if(N==1){capacitive=0; inductive=0;}//при рассмотрении одиночного ДП отключаютс параметры связи.
	F=sizeof(Ix)/sizeof(double);//вычисления количество ячеек
	G=sizeof(Itime1)/sizeof(double);//вычисления количество ячеек
	int pt[G];
	for(i=0; i<G; ++i){pt[i]=0;}
	
	//Массивы
	double *SumL, *SumC;//Сумма координатной производной второго порядка по всем контактам умноженная на обратная L
		SumL= new double [xN+2];
		SumC= new double [xN+2];
	double
		**P1,	**V1,	**ph,		**V,		**Q,		**dphx,	**dphxx,
		**P2,	**V2,	**int_ph_t,	**int_V_t,	**int_Q_t,	**int_dphx_t,
		**P3,	**V3,	**phav_t,	**Vav_t,	**Qav_t,	**dphxav_t,
		**P4,	**V4,	
		
		**Is,	**Iqp,	**Idisp,	**Idif,
		**V0,	**dV,	**dpht, **H,
		//;

	//---------
	 **PIs,**PIqp, **PIdisp, **PIdif;
	//---------
		//Коэффициенты Рунге-Кутта
		P1 = new double* [N+2];	V1 = new double* [N+2];
		P2 = new double* [N+2];	V2 = new double* [N+2];
		P3 = new double* [N+2];	V3 = new double* [N+2];
		P4 = new double* [N+2];	V4 = new double* [N+2];

		ph = new double* [N+2];			V = new double* [N+2];
		int_ph_t = new double* [N+2];	int_V_t = new double* [N+2];
		phav_t = new double* [N+2];		Vav_t = new double* [N+2];
				
		Q = new double* [N+2];			dphx = new double* [N+2];
		int_Q_t = new double* [N+2];	int_dphx_t = new double* [N+2];
		Qav_t = new double* [N+2];		dphxav_t = new double* [N+2];
		
		dphxx = new double* [N+2];
		
		
		
		Is = new double* [N+2];
		Iqp = new double* [N+2];
		Idisp = new double* [N+2];
		Idif = new double* [N+2];
		V0 = new double* [N+2];
		dV = new double* [N+2];
		dpht = new double* [N+2];


		//-----------------
		PIs = new double* [N+2];
		PIqp = new double* [N+2];
		PIdisp = new double* [N+2];
		PIdif = new double* [N+2];

		//-----------------


		
		H = new double* [N+2];
		
	for (int mi=0; mi<N+2;++mi){
		P1[mi] = new double [xN+2];	V1[mi] = new double [xN+2];
		P2[mi] = new double [xN+2];	V2[mi] = new double [xN+2];
		P3[mi] = new double [xN+2];	V3[mi] = new double [xN+2];
		P4[mi] = new double [xN+2];	V4[mi] = new double [xN+2];
		
		ph[mi] = new double [xN+2];			V[mi] = new double [xN+2];
		int_ph_t[mi] = new double [xN+2];	int_V_t[mi] = new double [xN+2];
		phav_t[mi] = new double [xN+2];		Vav_t[mi] = new double [xN+2];
		
		Q[mi] = new double [xN+2];			dphx[mi] = new double [xN+2];
		int_Q_t[mi] = new double [xN+2];	int_dphx_t[mi] = new double [xN+2];
		Qav_t[mi] = new double [xN+2];		dphxav_t[mi] = new double [xN+2];
		
		dphxx[mi] = new double [xN+2];
	
		
		
		Is[mi] = new double [xN+2];
		Iqp[mi] = new double [xN+2];
		Idisp[mi] = new double [xN+2];
		Idif[mi] = new double [xN+2];
		V0[mi] = new double [xN+2];
		dV[mi] = new double [xN+2];
		dpht[mi] = new double [xN+2];

	//----------------
		PIs[mi] = new double [xN+2];
		PIqp[mi] = new double [xN+2];
		PIdisp[mi] = new double [xN+2];
		PIdif[mi] = new double [xN+2];
		//------------
		
		H[mi] = new double [xN+2];
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//Начало процедуры создания матриц #####################################################	
	//Создание C
	if(capacitive==1){
		j=1;
		do{	i=1;
			do{C[i][j]=0;
				if (j==i) C[i][j]=DC;
				if (j==i-1) C[i][j]=SC;
				if (j==i+1) C[i][j]=SC;
			}while(++i<=N);
		}while(++j<=N);
		if (c=='p'){C[1][N]=SC; C[N][1]=SC;};
	}else{
		j=1;
		do{ i=1;
			do{	C[i][j]=0;
				if (j==i) C[i][j]=1;
			}while(++i<=N);
		}while(++j<=N);
	};
	//****************************************************************
	//Создание L
	if(inductive==1){
		j=1;
		do{	i=1;
			do{L[i][j]=0;
				if (j==i) {L[i][j]=1;}
				if (j==i-1) {L[i][j]=S;}
				if (j==i+1) {L[i][j]=S;}
			}while(++i<=N);
		}while(++j<=N);
		if (c=='p'){L[1][N]=S; L[N][1]=S;};
		//----------------------------------
		j=1;
		do{ i=1;
			do{	L1[i][j]=L[i][j];
			}while(++i<=N);
		}while(++j<=N);
		//---------------------------------------------
		//Создание единичной матрицы
		j=1;
		do{ i=1;
			do{	U[i][j]=0;
				if (j==i) U[i][j]=1;
			}while(++i<=N);
		}while(++j<=N);
		//----------------------------------------------
		//Нахождения обратной матрицы к L
		i=1;
		do{	r=L1[i][i];
			k=1;
			do{	 U[i][k]=U[i][k]/r;
				L1[i][k]=L1[i][k]/r;
			}while(++k<=N);
			j=1;
			do{	if (j!=i)
				{	pr=L1[j][i];
					k=1;
					do{
						U[j][k]=U[j][k]-U[i][k]*pr;
						L1[j][k]=L1[j][k]-L1[i][k]*pr;
					}while(++k<=N);
				}
			}while(++j<=N);
		}while(++i<=N);
		//-------------------------------------------------
		j=1;
		do{ i=1;
			do{	inversL[i][j]=U[i][j];
			}while(++i<=N);
		}while(++j<=N);
	}else{
		j=1;
		do{ i=1;
			do{	L[i][j]=0; inversL[i][j]=0;
				if (j==i){ L[i][j]=1; inversL[i][j]=1;}
			}while(++i<=N);
		}while(++j<=N);
	};
//Конец процедуры создания матриц ######################################################
//Начало процедуры создания файлов%%%%%%%%%%%%%%%%%%%%
	FILE *cvc,*cvcj,*cht,*vt,

	//*fvx,*fvxz,
	*fvxt,		
	//*fphx,*fphxz,
	*fphxt,		
	//*fdphx,	*fdphxz,
	*fdphxt,	
	//*fqx,	*fqxz,
	*fqxt,		
	
	*fIsxt, *fIqpxt,*fIdispxt,*fIdifxt,*fdphxxt,
	
	*fIs, *fIqp,*fIdisp,*fIdif,*fdpht,*fIsum;


	//Создание файлов
	//Передача создания файлов 0 процессору
	if (rank==0)
	{
	cvc=fopen("Voltage.dat","w");//создать файл для полной ВАХ
	cvcj=fopen("Voltage_j.dat","w");//создать файл для ВАХ в каждом контакте
	fdpht=fopen("dph_t.dat","w");
	if(currents==1){
		fIs=fopen("Is_j.dat","w");
		fIqp=fopen("Iqp_j.dat","w");
		fIdisp=fopen("Idisp_j.dat","w");
		fIdif=fopen("Idif_j.dat","w");
		fIsum=fopen("Isum.dat","w");//
	}
	if(time_dep==1){
		for ( g=0; g<G; ++g){			
			
				sprintf(str,"Ch_time_I_%.4f-%.4f.dat",Itime1[g],Itime2[g]);
				cht=fopen(str,"w");
				fclose(cht);
			
			sprintf(str,"Voltage_time_I_%.4f-%.4f.dat",Itime1[g],Itime2[g]);
			vt=fopen(str,"w");
			fclose(vt);		
		}
	}
	if(spatiotemporal_dep==1){
	
		
		for ( f=0; f<F; ++f){
			i=0;
			sprintf(str,"Q(t-x)_I-%f_layer-%i.dat",Ix[f],i);
			fqxt=fopen(str,"w");
			fclose(fqxt);	
		}
		
		for(i=1; i<=N; ++i){
			for ( f=0; f<F; ++f){
				sprintf(str,"H(t-x)_I-%f_junc-%i.dat",Ix[f],i);
				fdphxt=fopen(str,"w");
				fclose(fdphxt);
				
				sprintf(str,"Isurf(t-x)_I-%f_junc-%i.dat",Ix[f],i);
				fdphxxt=fopen(str,"w");
				fclose(fdphxxt);
				
				sprintf(str,"Phase(t-x)_I-%f_junc-%i.dat",Ix[f],i);
				fphxt=fopen(str,"w");
				fclose(fphxt);
				
				sprintf(str,"Voltage(t-x)_I-%f_junc-%i.dat",Ix[f],i);
				fvxt=fopen(str,"w");
				fclose(fvxt);

				sprintf(str,"Q(t-x)_I-%f_layer-%i.dat",Ix[f],i);
				fqxt=fopen(str,"w");
				fclose(fqxt);
				
			}

			if(currents==1){
				for ( f=0; f<F; ++f){
					sprintf(str,"Is(t-x)_I-%f_junc-%i.dat",Ix[f],i);
					fIsxt=fopen(str,"w");
					fclose(fIsxt);
					
					sprintf(str,"Iqp(t-x)_I-%f_junc-%i.dat",Ix[f],i);
					fIqpxt=fopen(str,"w");
					fclose(fIqpxt);
					
					sprintf(str,"Idisp(t-x)_I-%f_junc-%i.dat",Ix[f],i);
					fIdispxt=fopen(str,"w");
					fclose(fIdispxt);
					
					
					sprintf(str,"Idif(t-x)_I-%f_junc-%i.dat",Ix[f],i);
					fIdifxt=fopen(str,"w");
					fclose(fIdifxt);
					
				}
			}
		}
	}
	}

	// EZ

	MPI_Bcast(&f,1,MPI_INT,0,comm);

	// EZ

//Конец	процедуры создания файлов%%%%%%%%%%%%%%%%%%%%
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//Начальные условия, когда разность фаз отлична от нуля
	//Передача 0 процессору
	
	if(init_condition==1){
		if (rank==0)
		{

		ifstream infile;	//саязывать переменную "infile" с файлом
		infile.open("initial.dat");	//Открыть файл начальных условий для чтения
		double line[4];	//переменная для чтения файла
		x=0;
		do{	j=0;
			do{	infile>>line[j];	//Вывод первую строку файла в "line"
			}while(++j<=2);
			i=1;
			do{	ph[i][x]=line[1];	//Запись данных в ph
			}while(++i<=N);
		}while(++x<=xN);
		infile.close();//закрыт файл начальных условый
		
		}
		MPI_Bcast(&ph,(N+2)*(xN+2),MPI_DOUBLE,0,comm);
	}
	
	// Нулевые начальные условия
	if(currents==1){
		i=0;
		do{	int_Isav_t[i]=0;
			int_Iqpav_t[i]=0;
			int_Idispav_t[i]=0;
			int_Idifav_t[i]=0;
		}while(++i<=N);
	}
	i=0;
	do{	int_Vav_t[i]=0;
		int_dphtav_t[i]=0;
		x=0;
		do{	V[i][x]=0;
			int_V_t[i][x]=0;
			int_ph_t[i][x]=0;
			int_dphx_t[i][x]=0;
			if(init_condition==0){ph[i][x]=0;}
		}while(++x<=xN);
	}while(++i<=N+1);
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//$$$Начало процедуры генерации шума$$$$$$$$$$$$$$$$$$
	//Generation of noise for Windows 
	//Генерация шума для Windows
	/*long int ranmax=0,ran=0;
	long long x1;
	i=1;
	do{	asm("rdtsc" : "=A"(x1));
		srand(x1);
		ran=rand();
		if(ranmax<ran)ranmax=ran;
	}while(++i<=100);
	cout<<"random max is :"<<ranmax<<endl;
	float irand;
	int randa[2000];
	i=1;
	do{	irand=rand();
		irand=2*(irand/ranmax-0.5);
		ran=100*irand;
		randa[ran+100]=randa[ran+100]+1;
	}while(++i<=1000000);
	*/

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


	// Generation of noise for Linux 
	// Генерация шума для Linux


	float irand;
	char *a12;
	unsigned  i1,i2=1;
	long int ranmax=0, ran=0;
		float rand;
	int randa[2000];
	i=1;
	// for(int irank = 0; irank<np; irank++)
	{
	//	if(rank==irank)
		{
	do{	initstate(i1,a12,i2);
		ran=random();
		if(ranmax<ran)ranmax=ran;
	}while(++i<=100000);
	cout<<"random max is :"<<ranmax<<endl;

	i=1;
	do{	initstate(i1,a12,i2);
		rand=random();
		rand=2*(rand/ranmax-0.5);
		ran=100*rand;
		randa[ran+100]=randa[ran+100]+1;
	}while(++i<=1000000);
	}
	}
	// $$$Конец процедуры генерации шума$$$$$$$$$$$$$$$$$$


	double Pmax[N+2], Pmin[N+2], Ppmax[N+2], Ppmin[N+2], Pdphx[xN+2], Pph[xN+2], PV[xN+2], Psum[N+2], Psum1[N+2], Psum2[N+2], Psum3[N+2],Psum4[N+2];
			starttime= MPI_Wtime();
			//Опеределение блоков MPI
			//{
			int xxmin,xxmax,xxxmin,xxxmax;
			delta= (xN)/np;
            xmin= rank*delta;
			xxxmin=xmin;
            xmax = (rank+1)* delta;
            if(rank==(np-1))
            {
				xmax=xN;
				xxmax=xN+1;
			
			}
			else
			{
				xxmax=xmax;
			}
			

			if(rank==0)
			{
				xmin=2;
				xxmin=1;
			}
			else
			{
			xxmin=xmin;
			}


//%%%Начало цикла по току%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	do{	
		

		if((spatiotemporal_dep==1)&&(a==ax)){
			for(i=0; i<F; ++i){
				if(I<(Ix[i]+JP1/2)&&I>(Ix[i]-JP1/2)){f=i;}
			}
		}
		
	

		if((time_dep==1)&&(a==at)){
			for(i=0; i<G; ++i){
				if(I>(Itime1[i]-JP1/2)&&I<(Itime2[i]+JP1/2)){g=i;}
			}
		}
	
	

		if((time_dep==1)&&(I>(Itime1[g]-JP1/2))&&(I<(Itime2[g]+JP1/2))&&(a==at)&&(pt[g]==0)){
			pt[g]=1;		
			
			if (rank==0)
			{
				sprintf(str,"Ch_time_I_%.4f-%.4f.dat",Itime1[g],Itime2[g]);
				cht=fopen(str,"a");		
			
			sprintf(str,"Voltage_time_I_%.4f-%.4f.dat",Itime1[g],Itime2[g]);
			vt=fopen(str,"a");
			}
		}
		
	

		//Noise calculation for each JJ (for Windows)
		//Вычисления шума для каждого ДП (для Windows)
		i=1;
//		do{	irand=rand(); //EZ
		do{	irand=random(); //EZ
			irand=2*(irand/ranmax-0.5);
			Irand[i]=noismax*irand;
		}while(++i<=N);

		//-----------------------


	

		//&&&Начало цикла по времени&&&&&&&&&&&&&&&
		nt=0;
		t=0;
		do{	

	
			//Здесь сохраняется значение напряжения в предыдущем шаге по времени
			if(currents==1){
				i=1;
				do{	x=xxmin;
					do{	V0[i][x]=V[i][x];
					}while(++x<xxmax);
				}while(++i<=N);
			}

	

		//@@@Начало процедуры Рунге-Кутта@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



		//Первые коэффициенты Рунге-Кутта$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
			i=1;

			do{	//Левый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
				if(rank==0)// Левый край 0 процессору
				{
				SumL[1]=0;
				SumC[1]=0;
				k=1;
				do{	SumL[1]=SumL[1]+inversL[i][k]*(2*(ph[k][2]-ph[k][1])-2*Hext*XP);
					SumC[1]=SumC[1]+C[i][k]*V[k][1];
				}while(++k<=N);
				V1[i][1]=TP*(c1*SumL[1]-beta*SumC[1]-sin(ph[i][1])+I+Irand[i]);
				P1[i][1]=TP*SumC[1];
				}
				// конец вычисления левого края
				//Внутренние точки по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				//x=2;

				//if(rank==0)// 0-ой процессор начинает со второй точки
				//{
				//	xmin=2;
				//}
				x=xmin;
				// начало вычисления внутренних точек через MPI

				do{	SumL[x]=0;
					SumC[x]=0;
					k=1;
					do{	SumL[x]=SumL[x]
							+inversL[i][k]*(ph[k][x+1]-2*ph[k][x]+ph[k][x-1]);
						SumC[x]=SumC[x]+C[i][k]*V[k][x];
					}while(++k<=N);
					V1[i][x]=TP*(c1*SumL[x]-beta*SumC[x]-sin(ph[i][x])+I+Irand[i]);
					P1[i][x]=TP*SumC[x];
				}
				//while(++x<=xN-1);
				while(++x<xmax);


				//Правый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				if(rank==np-1)// передача правого края последнему процессору
				{
				SumL[xN]=0;
				SumC[xN]=0;
				k=1;
				do{	SumL[xN]=SumL[xN]+inversL[i][k]*(2*(ph[k][xN-1]-ph[k][xN])+2*Hext*XP);
					SumC[xN]=SumC[xN]+C[i][k]*V[k][xN];
				}while(++k<=N);
				V1[i][xN]=TP*(c1*SumL[xN]-beta*SumC[xN]-sin(ph[i][xN])+I+Irand[i]);
				P1[i][xN]=TP*SumC[xN];
				}
			}while(++i<=N);


			//пересылка значений
	
			if(rank!=np-1)
			{

		for (i=1;i<=N;i++)
	         {
		      Pmax[i]=P1[i][xmax-1];
		 }
				MPI_Sendrecv(&Pmax[1],N,MPI_DOUBLE,rank+1,33,&Ppmin[1],N, MPI_DOUBLE,rank+1,44,comm,&status);
				for(i=1;i<=N;i++)
					{
						P1[i][xmax]=Ppmin[i];
					}
			}
			if(rank!=0)
			{

		for (i=1;i<=N;i++)
	         {
		      	  Pmin[i]=P1[i][xmin];
		  
	         }
				MPI_Sendrecv(&Pmin[1],N,MPI_DOUBLE,rank-1,44,&Ppmax[1],N, MPI_DOUBLE,rank-1,33,comm,&status);
				for(i=1;i<=N;i++)
					{
						P1[i][xmin-1]=Ppmax[i];

					}
			}



		//Вторые коэффициенты Рунге-Кутта $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
			i=1;
			do{	//Левый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				if(rank==0)
				{
				SumL[1]=0;
				SumC[1]=0;
				k=1;
				do{	SumL[1]=SumL[1]+inversL[i][k]*
						(2*((ph[k][2]+P1[k][2]/2)-(ph[k][1]+P1[k][1]/2))-2*Hext*XP);
					SumC[1]=SumC[1]+C[i][k]*(V[k][1]+V1[k][1]/2);
				}while(++k<=N);
				V2[i][1]=TP*(c1*SumL[1]-beta*SumC[1]-sin(ph[i][1]+P1[i][1]/2)+I+Irand[i]);
				P2[i][1]=TP*SumC[1];
				}
				//Внутренние точки по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				//x=2;
				x=xmin;
				do{	SumL[x]=0;
					SumC[x]=0;
					k=1;
					do{	SumL[x]=SumL[x]+inversL[i][k]*
							((ph[k][x+1]+P1[k][x+1]/2)-2*(ph[k][x]+P1[k][x]/2)+(ph[k][x-1]+P1[k][x-1]/2));
						SumC[x]=SumC[x]+C[i][k]*(V[k][x]+V1[k][x]/2);
					}while(++k<=N);
					V2[i][x]=TP*(c1*SumL[x]-beta*SumC[x]-sin(ph[i][x]+P1[i][x]/2)+I+Irand[i]);
					P2[i][x]=TP*SumC[x];
				}
				//while(++x<=xN-1);
				while(++x<xmax);
				//Правый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				if(rank==np-1)
				{
				SumL[xN]=0;
				SumC[xN]=0;
				k=1;
				do{	SumL[xN]=SumL[xN]+inversL[i][k]*
						(2*((ph[k][xN-1]+P1[k][xN-1]/2)-(ph[k][xN]+P1[k][xN]/2))+2*Hext*XP);
					SumC[xN]=SumC[xN]+C[i][k]*(V[k][xN]+V1[k][xN]/2);
				}while(++k<=N);
				V2[i][xN]=TP*(c1*SumL[xN]-beta*SumC[xN]-sin(ph[i][xN]+P1[i][xN]/2)+I+Irand[i]);
				P2[i][xN]=TP*SumC[xN];
				}
			}while(++i<=N);

			//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!пересылка значений
		


			if(rank!=np-1)
			{

		for (i=1;i<=N;i++)
	         {
		      Pmax[i]=P2[i][xmax-1];
		}
				MPI_Sendrecv(&Pmax[1],N,MPI_DOUBLE,rank+1,333,&Ppmin[1],N, MPI_DOUBLE,rank+1,444,comm,&status);
				for(i=1;i<=N;i++)
					{
						P2[i][xmax]=Ppmin[i];
					}
			}
      
		if(rank!=0)
			{

	for (i=1;i<=N;i++)
	         {
		      	  Pmin[i]=P2[i][xmin];
	         }
				MPI_Sendrecv(&Pmin[1],N,MPI_DOUBLE,rank-1,444,&Ppmax[1],N, MPI_DOUBLE,rank-1,333,comm,&status);
				for(i=1;i<=N;i++)
					{
						P2[i][xmin-1]=Ppmax[i];
					}
			}


		//Третые коэффициенты Рунге-Кутта $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
			i=1;
			do{	//Левый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				if(rank==0)
				{
				SumL[1]=0;
				SumC[1]=0;
				k=1;
				do{	SumL[1]=SumL[1]+inversL[i][k]*(2*((ph[k][2]+P2[k][2]/2)-(ph[k][1]+P2[k][1]/2))-2*Hext*XP);
					SumC[1]=SumC[1]+C[i][k]*(V[k][1]+V2[k][1]/2);
				}while(++k<=N);
				V3[i][1]=TP*(c1*SumL[1]-beta*SumC[1]-sin(ph[i][1]+P2[i][1]/2)+I+Irand[i]);
				P3[i][1]=TP*SumC[1];
				}
				//Внутренние точки по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				//x=2;
				x=xmin;
				do{	SumL[x]=0;
					SumC[x]=0;
					k=1;
					do{	SumL[x]=SumL[x]
							+inversL[i][k]*((ph[k][x+1]+P2[k][x+1]/2)-2*(ph[k][x]+P2[k][x]/2)+(ph[k][x-1]+P2[k][x-1]/2));
						SumC[x]=SumC[x]+C[i][k]*(V[k][x]+V2[k][x]/2);
					}while(++k<=N);
					V3[i][x]=TP*(c1*SumL[x]-beta*SumC[x]-sin(ph[i][x]+P2[i][x]/2)+I+Irand[i]);
					P3[i][x]=TP*SumC[x];
				}
				//while(++x<=xN-1);
				while(++x<xmax);
				//Правый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				if(rank==np-1)
				{
				SumL[xN]=0;
				SumC[xN]=0;
				k=1;
				do{	SumL[xN]=SumL[xN]+inversL[i][k]*(2*((ph[k][xN-1]+P2[k][xN-1]/2)-(ph[k][xN]+P2[k][xN]/2))+2*Hext*XP);
					SumC[xN]=SumC[xN]+C[i][k]*(V[k][xN]+V2[k][xN]/2);
				}while(++k<=N);
				V3[i][xN]=TP*(c1*SumL[xN]-beta*SumC[xN]-sin(ph[i][xN]+P2[i][xN]/2)+I+Irand[i]);
				P3[i][xN]=TP*SumC[xN];
				}
			}while(++i<=N);

			//111111111111111111111111111111111111пересылка значений

			if(rank!=np-1)
			{

		for (i=1;i<=N;i++)
	         {
		      Pmax[i]=P3[i][xmax-1];
		  }
				MPI_Sendrecv(&Pmax[1],N,MPI_DOUBLE,rank+1,331,&Ppmin[1],N, MPI_DOUBLE,rank+1,441,comm,&status);
				for(i=1;i<=N;i++)
					{
						P3[i][xmax]=Ppmin[i];
					}
			}
			if(rank!=0)
			{
		for (i=1;i<=N;i++)
	         {
		     Pmin[i]=P3[i][xmin];
	         }
				MPI_Sendrecv(&Pmin[1],N,MPI_DOUBLE,rank-1,441,&Ppmax[1],N, MPI_DOUBLE,rank-1,331,comm,&status);
				for(i=1;i<=N;i++)
					{
						P3[i][xmin-1]=Ppmax[i];
					}
			}

		//Четвертые коэффициенты Рунге-Кутта $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
			i=1;
			do{	//Левый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				if(rank==0)
				{
				SumL[1]=0;
				SumC[1]=0;
				k=1;
				do{	SumL[1]=SumL[1]+inversL[i][k]*(2*((ph[k][2]+P3[k][2])-(ph[k][1]+P3[k][1]))-2*Hext*XP);
					SumC[1]=SumC[1]+C[i][k]*(V[k][1]+V3[k][1]);
				}while(++k<=N);
				V4[i][1]=TP*(c1*SumL[1]-beta*SumC[1]-sin(ph[i][1]+P3[i][1])+I+Irand[i]);
				P4[i][1]=TP*SumC[1];
				}
				//Внутренние точки по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				//x=2;
				x=xmin;
				do{	SumL[x]=0;
					SumC[x]=0;
					k=1;
					do{	SumL[x]=SumL[x]
							+inversL[i][k]*((ph[k][x+1]+P3[k][x+1])-2*(ph[k][x]+P3[k][x])+(ph[k][x-1]+P3[k][x-1]));
						SumC[x]=SumC[x]+C[i][k]*(V[k][x]+V3[k][x]);
					}while(++k<=N);
					V4[i][x]=TP*(c1*SumL[x]-beta*SumC[x]-sin(ph[i][x]+P3[i][x])+I+Irand[i]);
					P4[i][x]=TP*SumC[x];
				}
				//while(++x<=xN-1);
				while(++x<xmax);
				//Правый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				if(rank==np-1)
				{
				SumL[xN]=0;
				SumC[xN]=0;
				k=1;
				do{	SumL[xN]=SumL[xN]+inversL[i][k]*(2*((ph[k][xN-1]+P3[k][xN-1])-(ph[k][xN]+P3[k][xN]))+2*Hext*XP);
					SumC[xN]=SumC[xN]+C[i][k]*(V[k][xN]+V3[k][xN]);
				}while(++k<=N);
				V4[i][xN]=TP*(c1*SumL[xN]-beta*SumC[xN]-sin(ph[i][xN]+P3[i][xN])+I+Irand[i]);
				P4[i][xN]=TP*SumC[xN];
				}
			}while(++i<=N);



		//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
			i=1;
			do{	
				//x=1;
				x=xxmin;
				do{	ph[i][x]=ph[i][x]+(P1[i][x]+2*P2[i][x]+2*P3[i][x]+P4[i][x])/6;
					V[i][x]=V[i][x]+(V1[i][x]+2*V2[i][x]+2*V3[i][x]+V4[i][x])/6;
					///////////////////////////////////
					//if ((i==N)&&(t==tN)&&(x==30) )
					//{
						// printf("Volt %d %d %d  \n",x,t,rank);
					//}

					//////////////////////////////////
				}while(++x<xxmax);
				
				//while(++x<=xN);
			}while(++i<=N);
		//			if (t==tN) printf("Volt %f %d  \n",V[N][30], rank);

		//@@@Конец процедуры Рунге-Кутта@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

			//!!!!!!!!!!!!!!!!!!!!1пересылка значений
	
			if(rank!=np-1)
			{
		for (i=1;i<=N;i++)
	         {
		      Pmax[i]=ph[i][xmax-1];
		 }
				MPI_Sendrecv(&Pmax[1],N,MPI_DOUBLE,rank+1,332,&Ppmin[1],N, MPI_DOUBLE,rank+1,442,comm,&status);
				for(i=1;i<=N;i++)
					{
						ph[i][xmax]=Ppmin[i];
					}
			}
			if(rank!=0)
			{
		for (i=1;i<=N;i++)
	         {
		  Pmin[i]=ph[i][xmin];
	         }
				MPI_Sendrecv(&Pmin[1],N,MPI_DOUBLE,rank-1,442,&Ppmax[1],N, MPI_DOUBLE,rank-1,332,comm,&status);
				for(i=1;i<=N;i++)
					{
						ph[i][xmin-1]=Ppmax[i];
					}
			}


		//Начало усреднение по координате
			i=1;
			do{	//---Начало интегрирования по координате по методу симпсона----
				Psum[i]=0;
				if(rank==0)
				{
				//intxV[i]=V[i][1]+V[i][xN];
					Psum[i]=Psum[i]+V[i][1];    // EZ
				} 
				if(rank==np-1)    // EZ
                {
                 Psum[i]=Psum[i]+V[i][xN];    // EZ
                }
				//int_V_x[i]=V[i][1]+V[i][xN];//
				//x=2;
				//Паралельный блок
				if(rank==0)// 0-ой процессор начинает со второй точки
				{
					xmin=2;
				}
				x=xmin;
				do{	Psum[i]=Psum[i]+((pow(-1,x)+3)*V[i][x]);
				}while(++x<xmax);


				Psum[i]=Psum[i]*(XP/3);
				//int_V_x[i]=int_V_x[i]*(XP/3);
				//---Конец интегрирования по координате------------------------
				MPI_Reduce(&Psum[i],&int_V_x[i],1,MPI_DOUBLE,MPI_SUM,0,comm);

				if(rank==0)
				{
				Vav_x[i]=int_V_x[i]/length;//Усреднение по координате
				//int_V_x[i]=0;
				//Vav_x[i]=int_V_x[i]/length;//Усреднение по координате
				//Конец усреднения по координате
				//--------------------------------------------------------------------------
				if (t>ti){int_Vav_t[i]=Vav_x[i]*TP+int_Vav_t[i];}//Интегрирования по времени
				}
			}while(++i<=N);
		//###################################################
		//"""Начало процедуры вычисления производной разности фаз по времени
			i=1;
			//паралелить----------ПРОВЕРИТЬ правильность определения цикла!!!!!!----------------
			do{	//x=1;
			   
				x=xxmin;
				do{	//вычисление производной разности фаз по времени
					dpht[i][x]=0;
					
					k=1;
					do{	
						dpht[i][x]=dpht[i][x]+C[i][k]*V[k][x];
						
					}while(++k<=N);
				//}while(++x<=xN);
				  }while(++x<xxmax);


				Psum[i]=0;

				//Начало интегрирования по координате по методу симпсона
				//int_dpht_x[i]=dpht[i][1]+dpht[i][xN];//
				if(rank==0)
				{
					Psum[i]=Psum[i]+dpht[i][1];
				}
				if (rank==np-1)
				{
					Psum[i]=Psum[i]+dpht[i][xN];
				}
				//x=2;
				x=xmin;

				do{	
					//int_dpht_x[i]=int_dpht_x[i]+((pow(-1,x)+3)*dpht[i][x]);
					Psum[i]=Psum[i]+((pow(-1,x)+3)*dpht[i][x]);
				//}while(++x<=xN-1);
				}while(++x<xmax);

				//int_dpht_x[i]=int_dpht_x[i]*(XP/3);
				Psum[i]=Psum[i]*(XP/3);

				//Конец интегрирования по координате
				MPI_Reduce(&Psum[i],&int_dpht_x[i],1,MPI_DOUBLE,MPI_SUM,0,comm);
				if (rank==0)
				{
				dphtav_x[i]=int_dpht_x[i]/length;

				if (t>ti){int_dphtav_t[i]=int_dphtav_t[i]+dphtav_x[i]*TP;}
				}
			}while(++i<=N);

			//------------проверить правильность кода написанного выше-------------------



			//-----------------------------------
			//"""Конец процедуры вычисления производной разности фаз по времени
		//###Начало процедуры вычисления отдельных токов через ДП##########
			// параллелить-----------------
			if(currents==1){
				i=1;
				do{	
					Psum1[i]=0;
					Psum2[i]=0;
					Psum3[i]=0;
					Psum4[i]=0;
					
					//x=1;
					x=xxmin;

/*					if(rank==np-1)
					{
						xxxmax=xmax;
					}
					else
					{
					xxxmax=xmax-1;
					}
*/
					
					do{
						Is[i][x]=sin(ph[i][x]);//костыль!!!
					}while(++x<xxmax);

					x=xxmin;
					do{
						Iqp[i][x]=beta*V[i][x]; 
					}while(++x<xxmax);


					x=xxmin;
					do{	//вычисление производной разности фаз по времени
					/*	dpht[i][x]=0;
						k=1;
						do{	dpht[i][x]=dpht[i][x]+C[i][k]*V[k][x];
						}while(++k<=N);
					*/
						//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						//Вычисление сверхпроводящего тока
						//Is[i][x]=sin(ph[i][x]); //разослать всем
						//PIs[i][x]=sin(ph[i][x]);
						//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						//Вычисления квазичастичного тока
						//Iqp[i][x]=beta*V[i][x]; // разослать всем
						//PIqp[i][x]=beta*V[i][x];
						//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						//Вычисления тока смещения
						Idisp[i][x]=(V[i][x]-V0[i][x])/TP; //разослать всем
						//PIdisp[i][x]=(V[i][x]-V0[i][x])/TP;
						//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						//Вычисления диффузионного тока
						
							Idif[i][x]=beta*(dpht[i][x]-V[i][x]);
								//PIdif[i][x]=beta*(dpht[i][x]-V[i][x]);
	
					//}while(++x<=xN);
					}while(++x<xxmax);
					//-----------------------------------------
						


						//////////////////////////////////////////

					//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
					//Начало усреднения по координате
					//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
					//Начало интегрирования по координате по методу симпсона

					//int_Is_x[i]=Is[i][1]+Is[i][xN];//
					//int_Iqp_x[i]=Iqp[i][1]+Iqp[i][xN];//
					//int_Idisp_x[i]=Idisp[i][1]+Idisp[i][xN];//
					//int_Idif_x[i]=Idif[i][1]+Idif[i][xN];//
					if (rank==0)
					{
						Psum1[i]=Psum1[i]+Is[i][1];
						Psum2[i]=Psum2[i]+Iqp[i][1];
						Psum3[i]=Psum3[i]+Idisp[i][1];
						Psum4[i]=Psum4[i]+Idif[i][1];

					}
					if (rank==np-1)
					{
						Psum1[i]=Psum1[i]+Is[i][xN];
						Psum2[i]=Psum2[i]+Iqp[i][xN];
						Psum3[i]=Psum3[i]+Idisp[i][xN];
						Psum4[i]=Psum4[i]+Idif[i][xN];
					}

					//x=2;
					x=xmin;
					do{	/*int_Is_x[i]=int_Is_x[i]+((pow(-1,x)+3)*Is[i][x]);
						int_Iqp_x[i]=int_Iqp_x[i]+((pow(-1,x)+3)*Iqp[i][x]);
						int_Idisp_x[i]=int_Idisp_x[i]+((pow(-1,x)+3)*Idisp[i][x]);
						
							int_Idif_x[i]=int_Idif_x[i]+((pow(-1,x)+3)*Idif[i][x]);*/
						Psum1[i]=Psum1[i]+((pow(-1,x)+3)*Is[i][x]);
						Psum2[i]=Psum2[i]+((pow(-1,x)+3)*Iqp[i][x]);
						Psum3[i]=Psum3[i]+((pow(-1,x)+3)*Idisp[i][x]);

						Psum4[i]=Psum4[i]+((pow(-1,x)+3)*Idif[i][x]);



						
					//}while(++x<=xN-1);
					}while(++x<xmax);


					/*int_Is_x[i]=int_Is_x[i]*(XP/3);
					int_Iqp_x[i]=int_Iqp_x[i]*(XP/3);
					int_Idisp_x[i]=int_Idisp_x[i]*(XP/3);
					
						int_Idif_x[i]=int_Idif_x[i]*(XP/3);*/
					Psum1[i]=Psum1[i]*(XP/3);
					Psum2[i]=Psum2[i]*(XP/3);
					Psum3[i]=Psum3[i]*(XP/3);

					Psum4[i]=Psum4[i]*(XP/3);

					//------------------------------
					//Конец интегрирования по координате
					MPI_Reduce(&Psum1[i],&int_Is_x[i],1,MPI_DOUBLE,MPI_SUM,0,comm);
					MPI_Reduce(&Psum2[i],&int_Iqp_x[i],1,MPI_DOUBLE,MPI_SUM,0,comm);
					MPI_Reduce(&Psum3[i],&int_Idisp_x[i],1,MPI_DOUBLE,MPI_SUM,0,comm);
					MPI_Reduce(&Psum4[i],&int_Idif_x[i],1,MPI_DOUBLE,MPI_SUM,0,comm);
					//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						/// 0--------------------------------
					if (rank==0)
					{
					//Усреднение по координате
					Isav_x[i]=int_Is_x[i]/length;		
					Iqpav_x[i]=int_Iqp_x[i]/length;
					Idispav_x[i]=int_Idisp_x[i]/length;
					Idifav_x[i]=int_Idif_x[i]/length;
					//Конец усреднения по координате
					//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
					//Интегрирование по времени
					if (t>ti){
						int_Isav_t[i]=int_Isav_t[i]+Isav_x[i]*TP;
						int_Iqpav_t[i]=int_Iqpav_t[i]+Iqpav_x[i]*TP;
						int_Idispav_t[i]=int_Idispav_t[i]+Idispav_x[i]*TP;
						int_Idifav_t[i]=int_Idifav_t[i]+Idifav_x[i]*TP;
					}
					}
					//-------------------------------------------
				}while(++i<=N);
			}
			//###Конец процедуры вычисления отдельных токов через ДП###################
				
	
// MPI_Barrier(comm);
 //printf("Volt before spatio %d %d %d \n",t, rank, f);
 // MPI_Barrier(comm);
 //printf("!!!%f. %f, %f, %d, %d, %d\n ",I,Ix[f],JP1/2,a,ax,rank);
 
 if((spatiotemporal_dep==1)&&(I<(Ix[f]+JP1/2))&&(I>(Ix[f]-JP1/2))&&(a==ax)){

				// printf("!!!%f. %f, %f, %d, %d, %d\n ",I,Ix[f],JP1/2,a,ax,rank);
		 // MPI_Barrier(comm);
		 // printf("Volt after if spatio %d %d  \n",t, rank);

	// MPI_Barrier(comm);
 //if(t==tN) printf("Volt after spatio %f %d  \n",V[N][30], rank);
	// MPI_Barrier(comm);

				//Начало вычисления производной разность фаз по координате~~~~~~~~~~~
				//i=1;
				//do{	//Левый край по координате
				//	dphx[i][1]=(-3*ph[i][1]+4*ph[i][2]-ph[i][3])/(2*XP);
				//	//Внутрненные точки по координате
				//	x=2;
				//	do{	dphx[i][x]=(ph[i][x+1]-ph[i][x-1])/(2*XP);
				//	}while(++x<=xN-1);
				//	//Правый край по координате
				//	dphx[i][xN]=(3*ph[i][xN]-4*ph[i][xN-1]+ph[i][xN-2])/(2*XP);
				//}while(++i<=N);


				i=1;
				do{	//Левый край по координате

					//Левый край считает 0 процессор
					if(rank==0)
					{
					dphx[i][1]=(-3*ph[i][1]+4*ph[i][2]-ph[i][3])/(2*XP);
					}

					//Внутрненные точки по координате
					//if(rank==0)// 0-ой процессор начинает со второй точки
					//{
					//xmin=2;
					//}
					x=xmin;
					do{	dphx[i][x]=(ph[i][x+1]-ph[i][x-1])/(2*XP);
					}while(++x<xmax);

					//Правый край по координате
					//Правый край считает крайний процессор
					if(rank==np-1)
					{
					dphx[i][xN]=(3*ph[i][xN]-4*ph[i][xN-1]+ph[i][xN-2])/(2*XP);
					}
					for(x=1;x<=xN;x++)
					{
						Pph[x]=0;
					}
					/*for(x=xxmin;x<xxmax;x++)
					{
						Pph[x]=ph[i][x];
					}*/

					//if (t==tN) printf("Volt before reduce %f %d  \n",V[N][30], rank);

					//MPI_Reduce(&Pph[1],&ph[i][1],xN/*+2*/,MPI_DOUBLE,MPI_SUM,0,comm);
					//MPI_Reduce(&dphx[i][1],&Pdphx[1],xN/*+2*/,MPI_DOUBLE,MPI_SUM,0,comm);
					//MPI_Reduce(&V[i][1],&PV[1],xN/*+2*/,MPI_DOUBLE,MPI_SUM,0,comm);


					
					if(np>1)
					{

					///////////////////////////////////
					MPI_Gather(&ph[i][xxxmin],delta,MPI_DOUBLE,&Pph[0],delta,MPI_DOUBLE,0,comm);
					MPI_Gather(&dphx[i][xxxmin],delta,MPI_DOUBLE,&Pdphx[0],delta,MPI_DOUBLE,0,comm);
					MPI_Gather(&V[i][xxxmin],delta,MPI_DOUBLE,&PV[0],delta,MPI_DOUBLE,0,comm);

					if((rank==np-1)&&(xN>delta*np))
					{
						MPI_Send(&ph[i][np*delta],xN-delta*np+1,MPI_DOUBLE,0,112,comm);
						MPI_Send(&dphx[i][np*delta],xN-delta*np+1,MPI_DOUBLE,0,113,comm);
						MPI_Send(&V[i][np*delta],xN-delta*np+1,MPI_DOUBLE,0,114,comm);

					}

					if((rank==0)&&(xN>delta*np))
					{
						MPI_Recv(&Pph[np*delta],xN-delta*np+1,MPI_DOUBLE,np-1,112,comm,&status);
						MPI_Recv(&Pdphx[np*delta],xN-delta*np+1,MPI_DOUBLE,np-1,113,comm,&status);
						MPI_Recv(&PV[np*delta],xN-delta*np+1,MPI_DOUBLE,np-1,114,comm,&status);

					}

					////////////////////////////////////
					if (rank==0)
					{
						for(x=1;x<=xN/*+2*/;x++)
						{
							V[i][x]=PV[x];	
							dphx[i][x]=Pdphx[x];
							ph[i][x]=Pph[x];
							
							//printf("Volt %f %d %d ",V[i][x],i,x);
						}
						//printf("\n");
					
					}
					}//end of (np>1)

					///////////////////////////////////////////////
					//рассылка
						/*MPI_Reduce(&Is[i][1],&PIs[i][1],xN,MPI_DOUBLE,MPI_SUM,0,comm);
						MPI_Reduce(&Iqp[i][1],&PIqp[i][1],xN,MPI_DOUBLE,MPI_SUM,0,comm);
						MPI_Reduce(&Idisp[i][1],&PIdisp[i][1],xN,MPI_DOUBLE,MPI_SUM,0,comm);
						MPI_Reduce(&Idif[i][1],&PIdif[i][1],xN,MPI_DOUBLE,MPI_SUM,0,comm);*/

					if(np>1)
					{
						/////////////////////////////////////////
						MPI_Gather(&Is[i][xxxmin],delta,MPI_DOUBLE,&PIs[i][0],delta,MPI_DOUBLE,0,comm);
						MPI_Gather(&Iqp[i][xxxmin],delta,MPI_DOUBLE,&PIqp[i][0],delta,MPI_DOUBLE,0,comm);
						MPI_Gather(&Idisp[i][xxxmin],delta,MPI_DOUBLE,&PIdisp[i][0],delta,MPI_DOUBLE,0,comm);
						MPI_Gather(&Idif[i][xxxmin],delta,MPI_DOUBLE,&PIdif[i][0],delta,MPI_DOUBLE,0,comm);


						if((rank==np-1)&&(xN>delta*np))
					{
						MPI_Send(&Is[i][np*delta],xN-delta*np+1,MPI_DOUBLE,0,212,comm);
						MPI_Send(&Iqp[i][np*delta],xN-delta*np+1,MPI_DOUBLE,0,213,comm);
						MPI_Send(&Idisp[i][np*delta],xN-delta*np+1,MPI_DOUBLE,0,214,comm);
						MPI_Send(&Idif[i][np*delta],xN-delta*np+1,MPI_DOUBLE,0,215,comm);

					}

					if((rank==0)&&(xN>delta*np))
					{
						MPI_Recv(&PIs[i][np*delta],xN-delta*np+1,MPI_DOUBLE,np-1,212,comm,&status);
						MPI_Recv(&PIqp[i][np*delta],xN-delta*np+1,MPI_DOUBLE,np-1,213,comm,&status);
						MPI_Recv(&PIdisp[i][np*delta],xN-delta*np+1,MPI_DOUBLE,np-1,214,comm,&status);
						MPI_Recv(&PIdif[i][np*delta],xN-delta*np+1,MPI_DOUBLE,np-1,215,comm,&status);

					}
					}
					else
					{
						for(x=1;x<=xN/*+2*/;x++)
						{
							PIs[i][x]=Is[i][x];
							PIqp[i][x]=Iqp[i][x];
							PIdisp[i][x]=Idisp[i][x];
							PIdif[i][x]=Idif[i][x];
						}
					}
					//////////////////////////////////////////////



				}while(++i<=N);


// if (t==tN) printf("Volt after reduce %f %d  \n",V[N][30], rank);

//////////////////////
				//if (rank==0)
				//{
				//FILE *fuk;
				//do{
				//sprintf(str,"Volt%f-%i.dat",Ix[f],i);
				//fuk=fopen(str,"a");//
				//for (x=1;x<=xN;x=x+2)
				//{
				//fprintf(fuk,"%.15f\t",V[i][x]);// не правильная пересылка на нескольких процессах
				//}
				//fprintf(fuk,"\n");
				//
				//}while(++i<=N);
				//fclose(fuk);
///////////////////////
				//}

				//Конец вычисления производной разность фаз по координате~~~~~~~~~~~~~
				//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				//Начало вычисления магнитного поля в ДП
				if (rank==0)
				{
				i=1;
				do{	//Левый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
					H[i][1]=0;
					k=1;
					do{	H[i][1]=H[i][1]+inversL[i][k]*dphx[k][1];
					}while(++k<=N);
					//Внутренние точки по координате
					x=2;
					do{
						H[i][x]=0;
						k=1;
						do{	H[i][x]=H[i][x]+inversL[i][k]*dphx[k][x];
						}while(++k<=N);
					}while(++x<=xN-1);
					//Правый край по координате ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
					H[i][xN]=0;
					k=1;
					do{	H[i][xN]=H[i][xN]+inversL[i][k]*dphx[k][xN];
					}while(++k<=N);
				}while(++i<=N);
				//Конец вычисления магнитного поля в ДП
				//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				//Начало вычисления второй производной разность фаз по координате~~~~~~~~~~~
				i=1;
				do{	//Левый край по координате
					dphxx[i][1]=(-3*dphx[i][1]+4*dphx[i][2]-dphx[i][3])/(2*XP);
					//Внутрненные точки по координате
					x=2;
					do{	dphxx[i][x]=(dphx[i][x+1]-dphx[i][x-1])/(2*XP);
					}while(++x<=xN-1);
					//Правый край по координате
					dphxx[i][xN]=(3*dphx[i][xN]-4*dphx[i][xN-1]+dphx[i][xN-2])/(2*XP);
				}while(++i<=N);
				//Конец вычисления второй производной разность фаз по координате~~~~~~~~~~~~~
			//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				//Начало вычисления заряда в сверхпроводящих слоях
				i=1;
				do{	x=1;
					do{	Q[i][x]=Q0*(V[i+1][x]-V[i][x]);
					}while(++x<=xN);
				}while(++i<=N-1);
				//Вычисления заряда для граничных слоях
				if(c=='p'){//Периодические граничные условия 
					x=1;
					do{	Q[N][x]=Q0*(V[1][x]-V[N][x]);
						Q[0][x]=Q[N][x];
					}while(++x<=xN);
				}
				//Конец вычисления на сверхпроводящем слое
				//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 


				if(t==nt){
					nt=nt+deltaT;
					
					i=0;
					sprintf(str,"Q(t-x)_I-%f_layer-%i.dat",Ix[f],i);
					fqxt=fopen(str,"a");//
					
					x=1;
					nx=1;
					do{	if(x==nx){
							nx=nx+deltaX;
							fprintf(fqxt,"%.15f\t",Q[i][x]);
						}
					}while(++x<=xN);
				
					fprintf(fqxt,"\n");
					fclose(fqxt);
				
					i=1;
					do{	sprintf(str,"H(t-x)_I-%f_junc-%i.dat",Ix[f],i);
						//sprintf(str,"dphx(t-x)_I-%f_junc-%i.dat",Ix[f],i);
						fdphxt=fopen(str,"a");
					
						sprintf(str,"Isurf(t-x)_I-%f_junc-%i.dat",Ix[f],i);
						fdphxxt=fopen(str,"a");
					
						sprintf(str,"Phase(t-x)_I-%f_junc-%i.dat",Ix[f],i);
						fphxt=fopen(str,"a");//
					
						sprintf(str,"Voltage(t-x)_I-%f_junc-%i.dat",Ix[f],i);
						fvxt=fopen(str,"a");//
					
						sprintf(str,"Q(t-x)_I-%f_layer-%i.dat",Ix[f],i);
						fqxt=fopen(str,"a");//
					
						x=1;
						nx=1;
						do{	if(x==nx){
								nx=nx+deltaX;
								fprintf(fdphxt,"%.15f\t",H[i][x]);// не правильная пересылка на нескольких процессах???????
								//fprintf(fdphxt,"%.15f\t",dphx[i][x]);
								fprintf(fdphxxt,"%.15f\t",dphxx[i][x]);// не правильная пересылка на нескольких процессах???????
								fprintf(fphxt,"%.15f\t",ph[i][x]);
								fprintf(fvxt,"%.15f\t",V[i][x]);// не правильная пересылка на нескольких процессах
								
								fprintf(fqxt,"%.15f\t",Q[i][x]);
							}
						}while(++x<=xN);


	

						fprintf(fdphxt,"\n");
						fprintf(fdphxxt,"\n");
						fprintf(fphxt,"\n");
						fprintf(fvxt,"\n");
						fprintf(fqxt,"\n");
					
						fclose(fdphxt);
						fclose(fdphxxt);
						fclose(fphxt);
						fclose(fvxt);
						fclose(fqxt);
					}while(++i<=N);
				
					if(currents==1){
						i=1;
						do{	sprintf(str,"Is(t-x)_I-%f_junc-%i.dat",Ix[f],i);
							fIsxt=fopen(str,"a");
					
							sprintf(str,"Iqp(t-x)_I-%f_junc-%i.dat",Ix[f],i);// не правильная пересылка на нескольких процессах
							fIqpxt=fopen(str,"a");//
					
							sprintf(str,"Idisp(t-x)_I-%f_junc-%i.dat",Ix[f],i);
							fIdispxt=fopen(str,"a");//
					
		
								sprintf(str,"Idif(t-x)_I-%f_junc-%i.dat",Ix[f],i);
								fIdifxt=fopen(str,"a");//
				
							x=1;
							nx=1;
							do{	if(x==nx){
									nx=nx+deltaX;


									/*fprintf(fIsxt,"%.15f\t",Is[i][x]);
									fprintf(fIqpxt,"%.15f\t",Iqp[i][x]);
									fprintf(fIdispxt,"%.15f\t",Idisp[i][x]);
									fprintf(fIdifxt,"%.15f\t",Idif[i][x]);*/

									fprintf(fIsxt,"%.15f\t",PIs[i][x]);
									fprintf(fIqpxt,"%.15f\t",PIqp[i][x]);
									fprintf(fIdispxt,"%.15f\t",PIdisp[i][x]);
									fprintf(fIdifxt,"%.15f\t",PIdif[i][x]);


								}
							}while(++x<=xN);
						
							fprintf(fIsxt,"\n");
							fprintf(fIqpxt,"\n");
							fprintf(fIdispxt,"\n");
							fprintf(fIdifxt,"\n");
						
							fclose(fIsxt);
							fclose(fIqpxt);
							fclose(fIdispxt);
							fclose(fIdifxt);
						}while(++i<=N);
					}
				}
	

				//Начало процедуры интегрирования по времени~~~~~~~~~~~~~~~~~~~~
				if (t>ti){
					i=1;
					do{	x=1;
						do{	int_V_t[i][x]=V[i][x]*TP+int_V_t[i][x];
						}while(++x<=xN);
					}while(++i<=N);
				}
			}//close rank==0


// EZ
				if((np>1)&&(rank==0))
				{
					for(i=1;i<=N;i++)
					{
						for(x=xmax; x<=xN; x++)	
					{
						V[i][x] = 0;
						dphx[i][x] = 0;
					}
					for(x=xmax+1; x<=xN; x++)	
					{
						ph[i][x] = 0;
					}
					}
				}
// EZ


			}//close spatiotemporal_dep=1

			// MPI_Barrier(comm);
//			printf("after close if satio %d %d  \n",t,rank);

			//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
			//***Starting of time dependence calculation*******************************
			// Начало вычисления временных зависимостей
			
			if((time_dep==1)&&(I>(Itime1[g]-JP1/2))&&(I<(Itime2[g]+JP1/2))&&(a==at))
			{tN=Tf1/TP;Tf=Tf1;
//			printf("after if time_dep %d %d  \n",t,rank);			
			//Recording time Formula
				//Формула записываемого времени

				if(at==-1){time=t*TP-(TP*tN*(Itime2[g]-I)*at)/JP;}
				if(at==1){time=t*TP+(TP*tN*(I-Itime1[g])*at)/JP;}
				
				if(rank==0)
				{

				fprintf(vt,"%f\t",time);
				i=1;
				do{	fprintf(vt,"%.15f\t",Vav_x[i]);
				}while(++i<=N);		
				fprintf(vt,"%f\t\n",I);
				
				
					fprintf(cht,"%f\t",time);
					//Calculation of charge in s-layers
					//Начало вычисление заряда в сверхпроводящем слое
					i=1;
					do{	x=1;
						do{	Q[i][x]=Q0*(V[i+1][x]-V[i][x]);
						}while(++x<=xN);
					}while(++i<=N-1);
					if(c=='p'){
						x=1;
						do{	Q[N][x]=Q0*(V[1][x]-V[N][x]);
							Q[0][x]=Q[N][x];
						}while(++x<=xN);
					}
					//Конец вычисления заряда в сверхпроводящем слое
					//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
					//Начало усреднения по координате
					i=0;
					do{	//Начало интегрирования по координате по методу симпсона--------------------
						int_Q_x[i]=Q[i][1]+Q[i][xN];//
						x=2;
						do{	int_Q_x[i]=int_Q_x[i]+((pow(-1,x)+3)*Q[i][x]);
						}while(++x<=xN-1);
						int_Q_x[i]=int_Q_x[i]*(XP/3);
						//Конец интегрирования по координате----------------------------------------
						Qav_x[i]=int_Q_x[i]/length;//Усреднение по координате
						//Конец усреднения по координате
					}while(++i<=N);
					//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
				
					fprintf(cht,"%e\t",Qav_x[0]);
					i=1;
					do{	//Here will be recorded charge in all layers
						//Здесь записывается заряд на всех слоях
						fprintf(cht,"%e\t",Qav_x[i]);
					}while(++i<=N);
					fprintf(cht,"%f\t\n",I);
				} //close if rank = 0
			} //close time_dep=1
			else
			// this is for time_dep=0
			{tN=Tf0/TP;Tf=Tf0;};
			//***Ending of time dependence calculation*********************
			//***Конец вычисления временных зависимостей*******************	

		}while(++t<tN+1);
		//&&&Конец цикла по времени&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
		//Начало записи среднего значения отдельных токов через ДП
		if(currents==1){
			if(rank==0)
			{
			fprintf(fIs,"%f\t",I);
			fprintf(fIqp,"%f\t",I);
			fprintf(fIdisp,"%f\t",I);
			fprintf(fIdif,"%f\t",I);
			fprintf(fIsum,"%f\t",I);
		
			i=1;
			do{	Isav[i]=int_Isav_t[i]/(Tf-Ti); //
				int_Isav_t[i]=0;
				
				Iqpav[i]=int_Iqpav_t[i]/(Tf-Ti); //
				int_Iqpav_t[i]=0;
				
				Idispav[i]=int_Idispav_t[i]/(Tf-Ti); //
				int_Idispav_t[i]=0;
				
				
					Idifav[i]=int_Idifav_t[i]/(Tf-Ti); //
					int_Idifav_t[i]=0;
				
				
				Isum[i]=Idispav[i]+Isav[i]+Iqpav[i]+Idifav[i];// summirovaniya tokov
			
				fprintf(fIs,"%.15f\t",Isav[i]);
				fprintf(fIqp,"%.15f\t",Iqpav[i]);
				fprintf(fIdisp,"%.15f\t",Idispav[i]);
				fprintf(fIdif,"%.15f\t",Idifav[i]);
				fprintf(fIsum,"%.15f\t",Isum[i]);
			}while(++i<=N);
				
			fprintf(fIs,"\n");
			fprintf(fIqp,"\n");
			fprintf(fIdisp,"\n");
			fprintf(fIdif,"\n");
			fprintf(fIsum,"\n");
		} //end if rank==0
		} //end if currents=1

		//Конец записи среднего значения отдельных токов через ДП
		//Начало процедуры записи координатных зависимостей усредненное по времени	

		//Начало усреднение по времени$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
		
		
		if (rank==0)
		{ i=1; v=0;
		do{	Vav[i]=int_Vav_t[i]/(Tf-Ti);
			int_Vav_t[i]=0;
			v=v+Vav[i];//Вычисления сумарного напряжения
			dphtav[i]=int_dphtav_t[i]/(Tf-Ti);
			int_dphtav_t[i]=0;
		}while(++i<=N);
		
		//Конец усреднения по времени $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
		cout<<"\n\nI:"<<I<<"\tV:"<<v;//Вывод результатов на экран
		//Начало записи данных**************************
		fprintf(cvc,"%f\t%.15f\n",I,v);
		fprintf(cvcj,"%f\t%.15f\t",I,v);
		fprintf(fdpht,"%f\t",I);
		i=1;
		do{	fprintf(cvcj,"%.15f\t",Vav[i]);
			fprintf(fdpht,"%.15f\t",dphtav[i]);
		}while(++i<=N);
		fprintf(cvcj,"\n");
		fprintf(fdpht,"\n");
		} //end if rank=0
		
		MPI_Bcast(&v,1,MPI_DOUBLE,0,comm); // EZ !!!

		//Конец записи данных********************
		//Условие для изменения направлении тока~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		
		if (I>Imax)
		{	cout<<"\n\n Here is max of current so we go back"<<endl;
			a=-1;
		}
		//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		
		I=I+a*JP;//Счетчик тока
		
		//Здесь файлы для записи временных зависимостей закрываются дла данного интервала
		if((time_dep==1)&&(I<(Itime1[g]-JP1/2))&&(I>(Itime2[g]+JP1/2))&&(a==at)&&(pt[g]==1)){
			if(rank==0)
			{
			//pt[g]=1;		
			fclose(cht);
			fclose(vt);
			}
		}
		//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		if (I<0)break;// Условие для остановки вычисления
		
		if ((I<=Imore2+JP1/2)&&(I>=Imore1-JP1/2)&&(a==-1)&&(v>=0.05))
		{JP=JP1;
		// printf("\n rank, JP I= %d, %f, %f %f \n", rank, JP, I,v);
		}
		else
		{JP=JP0;
		// printf("\n else rank, JP,I= %d, %f, %f %f \n", rank, JP,I,v);
		}
	}while(I<=100);
	//%%%Конец цикла по току%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	//Здесь закрываются файлы
	if (rank==0)
	{
	fclose(cvc);	fclose(cvcj);	fclose(fdpht);
	//if(time_dep==1){ if(N!=1){fclose(cht);}	fclose(vt);}
	if(currents==1){
		fclose(fIs);
		fclose(fIqp);
		fclose(fIdisp);
		fclose(fIdif);
		fclose(fIsum);
	}
	}
	endtime= MPI_Wtime();
if (rank==0)
{
	printf("\n");
	printf("Посчитано за %f секунд \n",endtime-starttime);
	printf("Посчитано за %f минут \n",(endtime-starttime)/60);
}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Удаление динамических переменных
	delete [] P1;	delete [] V1;
	delete [] P2;	delete [] V2;
	delete [] P3;	delete [] V3;
	delete [] P4;	delete [] V4;
	
	delete [] ph;		delete [] V;		delete [] dphx;			delete [] Q;
	delete [] int_ph_t;	delete [] int_V_t;	delete [] int_dphx_t;	delete [] int_Q_t;
	delete [] phav_t;	delete [] Vav_t;	delete [] dphxav_t;		delete [] Qav_t;
	
	delete [] dphxx;	
	
	delete [] SumL;		delete [] SumC;		
	
	delete []Is;	delete []Iqp;	delete []Idisp;			delete []Idif;
	delete []V0;	delete []dV;	delete []dpht; delete []H;

MPI_Finalize();
return(0);
}